/*
 * KMap.h
 *
 *  Created on: Sep 22, 2008
 *      Author: Santi Ontan
 *      Description:
 *      	- The objects in the map are organized in a GRID
 *      	- Each cell of the grid represents a square of the map of some size (by default it's 128x128)
 *      	- Each cell contains the objects that have the center inside of the cell
 *      	- The grid is used to accelerate collision detection, and to know which are the objects that have to be considered
 *      	  during each game cycle
 *      	- Only those cells that are visible will be active, and only the objects in those cells will be considered
 *
 *		- Players go to layer 4
 *		- Weapons go to layer 3
 *		- enemies go to layer 2
 */

#ifndef KMAP_H_
#define KMAP_H_

#include "BList.h"
#include "Symbol.h"
#include "KObject.h"
#include "K3DObject.h"

#define NLAYERS	8

class GridCell {
	friend class Knightmare;
public:
	~GridCell() {
		int i;

		for(i=0;i<NLAYERS;i++) {
			m_objects[i].ExtractAll();
			m_passive_objects[i].ExtractAll();
		}
	}

	List<KObject> m_objects[NLAYERS];
	List<KObject> m_passive_objects[NLAYERS];
};


class EnemyWave {
public:
	EnemyWave(Symbol *type,int number) {
		m_type = type;
		m_number = number;
	}

	~EnemyWave() {
		delete m_type;
	}

	Symbol *m_type;
	int m_number;
};


class KMap {
	friend class Knightmare;
public:
	KMap(char *mapName, char *mapWavesName);
	~KMap();

	void draw(int x,int y,float zoom,float angle,GLTManager *GLTM,bool preserve_angle); // x,y are the coordinates of the map that will be drawn on the top-left corner of the screen
	void drawShadows(List<K3DObject> *objects,int x,int y,float vlx,float vly,float vlz,float depth);
	bool cycle(List<class VirtualController> *vcl, class Knightmare *game, class GLTManager *GLTM, SFXManager *SFXM);

	void addObject(int layer,KObject *o);
	void addActiveObject(int layer,KObject *o);
	void addPassiveObject(int layer,KObject *o);
	
	void deleteObject(KObject *o);
	void deletePassiveObject(KObject *o);
	
	KObject *collisionWithTypes(KObject *o, float offs_x, float offs_y, List<Symbol> *objectTypes);
	KObject *collisionWithType(KObject *o, float offs_x, float offs_y, Symbol *objectType);
	KObject *collision(KObject *o, float offs_x, float offs_y);
	
	// does not consider passive objects:
	List<KObject> *getObjectsByType(Symbol *type);
	
	float get_current_y();

private:

	void toGrid(int x,int y,int *gx,int *gy);
	int toGrid(int x,int y);

	float m_current_y;				// The current position to which the players have advanced in the map

	List<KObject> m_objects[NLAYERS];			// This is a list of All the objects in the map (huge list)
	List<KObject> m_passive_objects[NLAYERS];	// This is a list of All the passive objects (like grass, columns, etc) (even more huge)
	List<KObject> m_active_objects[NLAYERS];	// This is a list with just the objects that have to be considered in a particular instant of time
												// i.e. the players and enemies and objects in the screen

	List<KObject> m_characters;

	K3DObject *m_plane;	// This objet is just used for the shadows

	List<EnemyWave> m_waves;

	int m_tile_width, m_tile_height;
	int m_width, m_height;				// The size of the map in pixels
	int m_grid_width,m_grid_height;			// The grid cell size
	int m_grid_dimensions_x,m_grid_dimensions_y;
	GridCell **m_CG;						// The grid (to accelerate the collision computations)

	int m_tileset_n_tiles;
	GLTile **m_tileset_tiles;

	int m_cycle;

};

#endif /* KMAP_H_ */
